#!/usr/bin/perl -w

# v0.1
# simple script for regulation of the thinkpad x200 fan as required by sensors
# the default firmware-regulation doesnt ram down after rampups.
# This may harm your computer! No responsibility taken for anything!

use strict;
use Term::ANSIColor;
use POSIX qw(setsid);

my $observeonly=0;   # setting to 1 enables dryrun-mode, just monitoring
my $daemon=1;        # should we become a daemon

# all sources for sensor-values to be monitored. Using the 'sensors' binary is more
# generic but also wastes more resources
my @sensorsrc=qw@	/sys/class/hwmon/hwmon0/temp1_input		/sys/class/hwmon/hwmon0/temp2_input		
		        /sys/class/hwmon/hwmon1/device/fan1_input 
		        /sys/class/hwmon/hwmon1/device/temp2_input 	/sys/class/hwmon/hwmon1/device/temp4_input
		        /sys/class/hwmon/hwmon1/device/temp5_input	/sys/class/hwmon/hwmon1/device/temp7_input
		        /sys/class/hwmon/hwmon1/device/temp9_input	/sys/class/hwmon/hwmon1/device/temp10_input	@;

my ( $sensor, $result, $i, @resu );    # runvars and outputbuffering
my ( $f1, $fanlevel, @sensors );       # monitored values
my ( $rampup, $rampdown, $rampupintend ); # votes for fan-rampup/down
                                          # rampupintend requires 2 sensor-runs to for for rampup,
                                          # preventing rampups from monitored temperature-spikes
my @names=( "cpu0","cpu1","pci/wlan","hdd","bat0","bat1","gpu","pciexpress"); # asumed sensorlocations
my @limits=( 44,    42,    49,        46,   37,    36,    50,   48);          # sensor-limits requesting rampup


$observeonly || do {
    @resu=qx@ echo 'level 1' >/proc/acpi/ibm/fan @;    # take over control
};

sub readsensors {
foreach $sensor (@sensorsrc) {
	open HNDL,"<$sensor" || die "Cant open $sensor for reading.$!\n";
	if ( $sensor =~ m/fan/ ) {
		$f1=<HNDL>;
		chomp($f1);
	}
	else {
		push(@sensors,<HNDL>/1000);
	}
	close(HNDL);
}
}

sub getfanstatus {
	open HNDL,"</proc/acpi/ibm/fan" || die "Cant open /proc/acpi/ibm/fan for reading.$!\n";
	@resu=<HNDL>;
	close HNDL;
	if ( $resu[2]=~m/([0-9])$/ ) {
		$fanlevel=$1;
	}
	else {
		$fanlevel="UKN";
	}
	if ( $resu[0] =~ m/ disabled / ) { print "<firmwaremode> fanlevel:$fanlevel "; }
	else { print "<usermode> fanlevel:$fanlevel "; }
}

$daemon && &daemonize;

while (1) {
	getfanstatus();

	$rampup=0;
	$rampdown=9;

	$#sensors = -1;
	readsensors();
	for ($i=0;$i<=$#sensors;$i++) {
		if ( $sensors[$i] <= $limits[$i]-2 ) {
			print color 'blue';
			print $limits[$i]-2, "/$sensors[$i]/$limits[$i] ";
			print color 'reset';
			$rampdown--;
		}
		if ( ( $sensors[$i] > $limits[$i]-2 ) && ( $sensors[$i] <= $limits[$i] ) ) {
			print $limits[$i]-2, "/$sensors[$i]/$limits[$i] ";
		}
		if ( $sensors[$i] > $limits[$i] ) {
			print color 'red';
			print $limits[$i]-2, "/$sensors[$i]/$limits[$i] ";
			print color 'reset';
			$rampup=1;
		}
	}
	print "rampup/down: $rampup/$rampdown ";
	if ( $rampup == 1 ) {
	    if ( $fanlevel<4 ) {
		if ( $rampupintend == 0 ) {
		    $rampupintend=1;
		    print "rampupintend=1 ";
		}
		else {
		    $rampupintend=0;
		    $fanlevel++; 
		    $observeonly || do {
			$result=qx@ echo "level $fanlevel" >/proc/acpi/ibm/fan @;
		    };
		    print "rampup ";
		}
	    }
	}
	else {
	    $rampupintend=0;
	}

	if ( ($rampdown == 1) && ($fanlevel>1) ) {
	    $fanlevel--; 
	    $observeonly || do {
		$result=qx@ echo "level $fanlevel" >/proc/acpi/ibm/fan @;
	    };
	    print "rampdown ";
	}
	    
	print "\n";

	sleep 5;
}

sub daemonize {
    chdir '/'                 or die "Can't chdir to /: $!";
    open STDIN, '/dev/null'   or die "Can't read /dev/null: $!";
    open STDOUT, '>>/dev/null' or die "Can't write to /dev/null: $!";
    open STDERR, '>>/dev/null' or die "Can't write to /dev/null: $!";
    defined(my $pid = fork)   or die "Can't fork: $!";
    exit if $pid;
    setsid                    or die "Can't start a new session: $!";
    umask 0;
}
